@@ -32,7 +32,7 @@
 #endif
 #endif
 
-#define COPYBLURB "Copyright (c) 2002-2007 L.A. Breyer. All rights reserved.\n" \
+#define COPYBLURB "Copyright (c) 2002-2013 L.A. Breyer. All rights reserved.\n" \
                   "%s comes with ABSOLUTELY NO WARRANTY, and is licensed\n" \
 	          "to you under the terms of the GNU General Public License 3 or later.\n\n"
 
@@ -172,17 +172,18 @@ typedef unsigned int token_class_t; /* used in bit-field, therefore uint */
 typedef u_int8_t hash_bit_count_t;
 typedef u_int64_t hash_count_t;
 typedef unsigned int hash_percentage_t;
-typedef u_int8_t category_count_t;
-typedef u_int8_t regex_count_t;
-typedef u_int32_t document_count_t;
+typedef u_int16_t category_count_t;
+typedef u_int16_t regex_count_t;
+typedef u_int64_t document_count_t;
 typedef u_int16_t confidence_t;
 
 typedef float weight_t;
-typedef long double score_t;
-#define FMT_printf_score_t "Lf"
-#define FMT_scanf_score_t "Lf"
+typedef double score_t;
+#define FMT_printf_score_t "f"
+#define FMT_scanf_score_t "lf"
+#define FMT_printf_integer_t "ld"
 
-typedef u_int8_t token_stack_t;
+typedef u_int16_t token_stack_t;
 typedef int charbuf_len_t;
 typedef u_int16_t alphabet_size_t;
 typedef u_int16_t smbitmap_t;
@@ -211,7 +212,7 @@ typedef u_int16_t digitized_weight_t;
 /* size of hash in bits */
 #define MAX_HASH_BITS ((hash_bit_count_t)64)
 /* for line filtering: maximum number of tokens allowed on a single line */
-#define MAX_TOKEN_LINE_STACK ((token_stack_t)255) 
+#define MAX_TOKEN_LINE_STACK ((token_stack_t)16384) 
 /* number of pages we want to use for I/O buffering */
 #define BUFFER_MAG 64
 /* we need 8 byte hash values */
@@ -235,6 +236,7 @@ typedef float weight_t;
 typedef double score_t;
 #define FMT_printf_score_t "f"
 #define FMT_scanf_score_t "lf"
+#define FMT_printf_integer_t "d"
 
 typedef u_int8_t token_stack_t;
 typedef int charbuf_len_t;
@@ -268,8 +270,8 @@ typedef u_int16_t digitized_weight_t;
 /* number of pages we want to use for I/O buffering */
 #define BUFFER_MAG 32
 /* we need 4 byte hash values */
-#undef  JENKINS8
-#define JENKINS4
+#define  JENKINS8
+#undef JENKINS4
 
 #elif defined SMALL_MEMORY_MODEL
 
@@ -288,6 +290,7 @@ typedef float weight_t;
 typedef double score_t;
 #define FMT_printf_score_t "f"
 #define FMT_scanf_score_t "lf"
+#define FMT_printf_integer_t "d"
 
 typedef u_int8_t token_stack_t;
 typedef int charbuf_len_t;
@@ -343,6 +346,7 @@ typedef float weight_t;
 typedef double score_t;
 #define FMT_printf_score_t "f"
 #define FMT_scanf_score_t "lf"
+#define FMT_printf_integer_t "d"
 
 typedef u_int8_t token_stack_t;
 typedef int charbuf_len_t;
@@ -402,7 +406,7 @@ typedef u_int16_t digitized_weight_t;
 #define DIGITIZED_WEIGHT_MAX ((digitized_weight_t)USHRT_MAX)
 #define DIG_FACTOR           5
 /* maximum number of categories we can handle simultaneously */
-#define MAX_CAT ((category_count_t)64)
+#define MAX_CAT ((category_count_t)16383)
 /* percentage of hash we use */
 #define HASH_FULL ((hash_percentage_t)95)
 /* alphabet size */
@@ -676,7 +680,7 @@ typedef enum {gcUNDEF = 0, gcDISCARD, gcTOKEN, gcTOKEN_END, gcIGNORE} good_char_
 #define MAGIC_ONLINE "# dbacl " SIGNATURE " online memory dump\n"
 
 #define MAGIC_DUMP "# lambda | dig_ref | count | id     | token\n"
-#define MAGIC_DUMPTBL_o "%9.3f %9.3f %7d %8lx "
+#define MAGIC_DUMPTBL_o "%9.3f %9.3f %7" FMT_printf_integer_t " %8lx "
 #define MAGIC_DUMPTBL_i "%f %f %d %lx "
 
 /* data structures */
